@@ -33,12 +33,12 @@
  * SUCH DAMAGE.
  */
 
-#ifndef lint
-static const char rcsid[] = "$Id: priv_mktemp.c,v 1.2 2001/10/01 00:21:34 millert Exp $";
-#endif /* not lint */
-
 #include "config.h"
 
+#ifndef lint
+static const char rcsid[] = "$Id: priv_mktemp.c,v 1.7 2003/03/24 01:08:22 millert Exp $";
+#endif /* not lint */
+
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <fcntl.h>
@@ -54,19 +54,7 @@ static const char rcsid[] = "$Id: priv_mktemp.c,v 1.2 2001/10/01 00:21:34 miller
 
 #include <extern.h>
 
-static int _gettemp __P((char *, int *, int, int));
-
-#if 0
-int
-mkstemps(path, slen)
-	char *path;
-	int slen;
-{
-	int fd;
-
-	return (_gettemp(path, &fd, 0, slen) ? fd : -1);
-}
-#endif
+static int _gettemp __P((char *, int *, int));
 
 int
 priv_mkstemp(path)
@@ -74,54 +62,45 @@ priv_mkstemp(path)
 {
 	int fd;
 
-	return (_gettemp(path, &fd, 0, 0) ? fd : -1);
+	return (_gettemp(path, &fd, 0) ? fd : -1);
 }
 
 char *
 priv_mkdtemp(path)
 	char *path;
 {
-	return(_gettemp(path, (int *)NULL, 1, 0) ? path : (char *)NULL);
+	return(_gettemp(path, (int *)NULL, 1) ? path : (char *)NULL);
 }
 
 static int
-_gettemp(path, doopen, domkdir, slen)
+_gettemp(path, doopen, domkdir)
 	char *path;
 	register int *doopen;
 	int domkdir;
-	int slen;
 {
-	register char *start, *trv, *suffp;
+	register char *start, *trv;
 	struct stat sbuf;
 	int pid, rval;
+	char *alphabet = "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz";
 
 	if (doopen && domkdir) {
 		errno = EINVAL;
 		return(0);
 	}
 
+	pid = getpid();
 	for (trv = path; *trv; ++trv)
 		;
-	trv -= slen;
-	suffp = trv;
 	--trv;
-	if (trv < path) {
-		errno = EINVAL;
-		return (0);
-	}
-	pid = getpid();
-	while (*trv == 'X' && pid != 0) {
+	while (trv >= path && *trv == 'X' && pid != 0) {
 		*trv-- = (pid % 10) + '0';
 		pid /= 10;
 	}
-	while (*trv == 'X') {
+	while (trv >= path && *trv == 'X') {
 		char c;
 
 		pid = (get_random() & 0xffff) % (26+26);
-		if (pid < 26)
-			c = pid + 'A';
-		else
-			c = (pid - 26) + 'a';
+		c = alphabet[pid];
 		*trv-- = c;
 	}
 	start = trv + 1;
@@ -168,19 +147,35 @@ _gettemp(path, doopen, domkdir, slen)
 		for (trv = start;;) {
 			if (!*trv)
 				return (0);
-			if (*trv == 'Z') {
-				if (trv == suffp)
-					return (0);
+			if (*trv == 'Z')
 				*trv++ = 'a';
-			} else {
+			else {
 				if (isdigit((unsigned char)(*trv)))
 					*trv = 'a';
-				else if (*trv == 'z')	/* inc from z to A */
+				else if (*trv == 'z')	/* wrap from z to A */
 					*trv = 'A';
 				else {
-					if (trv == suffp)
-						return (0);
+#ifdef HAVE_EBCDIC
+					switch(*trv) {
+					case 'i':
+						*trv = 'j';
+						break;
+					case 'r':
+						*trv = 's';
+						break;
+					case 'I':
+						*trv = 'J';
+						break;
+					case 'R':
+						*trv = 'S';
+						break;
+					default:
+						++*trv;
+						break;
+					}
+#else
 					++*trv;
+#endif
 				}
 				break;
 			}
