@@ -38,18 +38,19 @@
 #endif /* HAVE_STDLIB_H */
 #ifdef HAVE_STRING_H
 # include <string.h>
+#else
+# ifdef HAVE_STRINGS_H
+#  include <strings.h>
+# endif /* HAVE_STRINGS_H */
 #endif /* HAVE_STRING_H */
-#ifdef HAVE_STRINGS_H
-# include <strings.h>
-#endif /* HAVE_STRINGS_H */
-#if defined(HAVE_MALLOC_H) && !defined(HAVE_STDLIB_H)
+#if defined(HAVE_MALLOC_H) && !defined(STDC_HEADERS)
 # include <malloc.h>
-#endif /* HAVE_MALLOC_H && !HAVE_STDLIB_H */
+#endif /* HAVE_MALLOC_H && !STDC_HEADERS */
 
 #include <extern.h>
 
 #ifndef lint
-static const char rcsid[] = "$Id: strdup.c,v 1.2 2001/10/01 00:21:34 millert Exp $";
+static const char rcsid[] = "$Id: strdup.c,v 1.3 2001/11/12 20:02:06 millert Exp $";
 #endif /* lint */
 
 char *
